@@ -21,7 +21,7 @@ from . import eduperson
 from . import backend
 from .config import CONFIG
 from .results import Deployed, NotDeployed, Rejection, Failure, Question, raise_question, Status
-from .name_generators import FriendlyNameGenerator
+from .name_generators import FriendlyNameGenerator, PooledNameGenerator
 
 logger = logging.getLogger(__name__)
 
@@ -386,6 +386,7 @@ class User:
 
             # Raise question in case of existing username in case we're interactive
             if CONFIG.getboolean('ldf_adapter', 'interactive', fallback=False): # interactive
+                logger.debug("interactive mode")
                 if self.service_user.name_taken(username):
                     logger.info(F'Username "{username}" is already taken, asking user to pick a new one')
                     raise Question(
@@ -394,19 +395,33 @@ class User:
                     )
 
             else: # non-interactive
-                try:
-                    logger.info(F"initial try: {username}")
-                except AttributeError:
-                    pass
-                fng = FriendlyNameGenerator(self.data)
-                proposed_name = username if username is not None else fng.suggest_name()
+                logger.debug("noninteractive mode")
+                username_mode = CONFIG.get('username_generator','mode', fallback='friendly')
+                logger.debug(F"username_mode: {username_mode}")
+                if username_mode == 'friendly':
+                    try:
+                        logger.info(F"Trying username: {username}")
+                    except AttributeError:
+                        pass
+                    name_generator = FriendlyNameGenerator(self.data)
+                    # Propose an initial name
+                    proposed_name = username if username is not None else name_generator.suggest_name()
+
+                elif username_mode == 'pooled':
+                    primary_group_name = self.data.primary_group
+                    name_generator = PooledNameGenerator(pool_prefix = primary_group_name)
+                    # Propose an initial name
+                    proposed_name = name_generator.suggest_name()
+
                 while self.service_user.name_taken(proposed_name):
-                    proposed_name = fng.suggest_name()
+                    proposed_name = name_generator.suggest_name()
                 logger.info(F'Using: {proposed_name}')
                 if proposed_name is None:
                     raise Rejection(message=F"I cannot create usernames. "
-                                    F"The list of tried ones is: {', '.join(fng.tried_names())}.")
+                                    F"The list of tried ones is: {', '.join(name_generator.tried_names())}.")
                 self.service_user.set_username(proposed_name)
+
+
             self.service_user.create()
         else: # The user exists
             # Update service_user.name if unique_id already points to a username:
@@ -640,7 +655,7 @@ class UserInfo(Mapping):
         sub = regex.sub('[^a-zA-Z0-9_!#$%&*+/=?{|}~^.-]', '-', sub)
 
         if sub != self.userinfo['sub']:
-            logger.warning("sub '{}' changed to '{}' for BWIDM compatibilty".format(
+            logger.warning("sub '{}' changed to '{}' for general compatibilty".format(
                 self.userinfo['sub'], sub))
         return sub
 
@@ -659,7 +674,7 @@ class UserInfo(Mapping):
         # and there shouldn't be two different issuers `http://example.org' and `https://example.org'.
         if iss != stripped_iss:
             if CONFIG.getboolean('messages', 'log_name_changes', fallback=True):
-                logger.warning("Issuer '{}' changed to '{}' for BWIDM compatibilty".format(
+                logger.warning("Issuer '{}' changed to '{}' for general compatibilty".format(
                     stripped_iss, iss))
         return iss
 
@@ -763,6 +778,14 @@ class UserInfo(Mapping):
 
         Group names are prefixed with the delegated namespace from the entitlement.
         """
+        if CONFIG.getboolean('username_generator', 'strip_sub_groups', fallback=False):
+            logger.debug("Stripping all subgroups")
+            return set(filter(
+                None,
+                ['{}_{}'.format(ns, grp) for (ns, grp) in chain.from_iterable(
+                     (("-".join([ent.delegated_namespace] + ent.subnamespaces), grp) for grp in ent.all_toplevel_groups)
+                     for ent in self.entitlement)]
+            ))
         return set(filter(
             None,
             ['{}_{}'.format(ns, grp) for (ns, grp) in chain.from_iterable(
@@ -802,7 +825,7 @@ class UserInfo(Mapping):
 
         if grp != orig_grp:
             if CONFIG.getboolean('messages', 'log_name_changes', fallback=True):
-                logger.warning("Group name '{}' changed to '{}' for BWIDM compatibilty".format(orig_grp, grp))
+                logger.warning("Group name '{}' changed to '{}' for general compatibilty".format(orig_grp, grp))
 
         return grp
 
