@@ -1,3 +1,7 @@
+# vim: foldmethod=indent : tw=100
+# pylint: disable=invalid-name, superfluous-parens
+# pylint: disable=logging-fstring-interpolation, logging-not-lazy, logging-format-interpolation
+# pylint: disable=missing-docstring, too-few-public-methods
 """
 BWIDM backend.
 
@@ -18,6 +22,7 @@ from ..config import CONFIG
 from .. import utils
 from time import sleep
 
+from ldf_adapter.logsetup import jsonlogger
 logger = logging.getLogger(__name__)
 
 class BwIdmConnection:
@@ -80,6 +85,7 @@ class User:
         self.info = userinfo
         self.credentials = {}
         self.primary_group = Group(userinfo.primary_group)
+        self.force_username = None
 
     def exists(self):
         """
@@ -100,32 +106,62 @@ class User:
             self.info.unique_id, status
         ))
         return status == self.VALUE_USER_ACTIVE
+    def _is_registered(self):
+        """
+        find if the user is already registered for a given
+        service, identified by its service short name
+        """
+        # FIXME: Consider putting this request into the global user object (to reduce load on regapp)
+        ssn = CONFIG['backend.bwidm.service']['name']
+        registrations = BWIDM.get ('external-reg', 'find', 'externalId',
+                                    self.info.unique_id)
+        # find registrations
+        number_of_registrations = 0
+        try:
+            logger.debug("logging registrations to jsonlog")
+            jsonlogger.debug(json.dumps(registrations.json, sort_keys=True, indent=4, separators=(',', ': ')))
+        except TypeError:
+            pass
+        for reg in registrations.json():
+            if reg["serviceShortName"] == ssn:
+                if reg["registryStatus"] == "ACTIVE":
+                    number_of_registrations += 1
+        logger.debug(F"Number of registrations found: {number_of_registrations}")
+        if number_of_registrations > 0:
+            return True
+        return False
 
-    def name_taken(self):
+    def name_taken(self, name):
         """
         If there is a user for our unique_id with our username, treat the name as available. This
         might happen if the our user is ON_HOLD on the service.
         """
+        # TODO: argument "name" was added, check if given "name" (instead of info.username) is taken by *another* user
+        logger.debug(F"checking for name taken of {name}")
         users_with_name = BWIDM.get(
             'external-user', 'find',
-            'attribute', self.ATTR_USERNAME, self.info.username
+            'attribute', self.ATTR_USERNAME, name
         ).json()
 
         other_users_with_name = [user for user in users_with_name if user['externalId'] != self.info.unique_id]
+        # logger.debug (F"other_users: {other_users_with_name}")
+        logger.debug(F"Found {len(other_users_with_name)} with same username")
+        # FIXME: Why < ?
         if len(other_users_with_name) < len(users_with_name):
             logger.debug("Username '{}' is reserved for us".format(self.info.username))
 
         if other_users_with_name:
-            logger.error("Username '{}' is already used by {}".format(
+            logger.error("Username '{}' is already used by\n    {}".format(
                 self.info.username,
-                ", ".join(map(lambda u: u['externalId'], other_users_with_name))
+                ",\n    ".join(map(lambda u: u['externalId'], other_users_with_name))
             ))
         else:
-            logger.debug("Username '{}' is available".format(self.info.username))
+            logger.debug("Username '{}' is available".format(name))
 
         return bool(other_users_with_name)
 
     def get_username(self):
+        """Check if a user exists based on unique_id and return the name"""
         def safe_resp_conversion(resp):
             '''Safely convert a response to json'''
             if resp.status_code != 200:
@@ -137,11 +173,8 @@ class User:
                 logging.error ('Could not decode json that I obtained from rest server')
                 raise
             return resp_json
-
-        """Check if a user exists based on unique_id"""
         full_username = None
-        external_id   = self.info.unique_id
-        resp          = BWIDM.get ('external-user', 'find', 'externalId', external_id)
+        resp          = BWIDM.get ('external-user', 'find', 'externalId', self.info.unique_id)
         resp_json     = safe_resp_conversion(resp)
 
         try:
@@ -152,11 +185,15 @@ class User:
             logger.error('Error: I could not find the username in the database.')
             logger.error('  Most likely the user is not registered for this service\n')
             logger.error(F"  {e}")
-            logger.error('  This is the json data received\n')
             logger.error(json.dumps(resp_json, sort_keys=True, indent=4, separators=(',', ': ')))
         logger.debug(F"Found existing username: {full_username}")
         return full_username
 
+    def set_username(self, username):
+        """Update the internal representation of the user with the incoming username"""
+        logger.debug("aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa")
+        self.force_username = username
+        logger.debug("bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb")
 
     def create(self):
         """Create or activate user."""
@@ -170,6 +207,7 @@ class User:
             })
 
     def update(self):
+        # FIXME: Consider putting this request into the global user object (to reduce load on regapp)
         def get_active_reg_info(ext_id):
             rsp = BWIDM.get('external-reg', 'find',
                             'externalId', ext_id)
@@ -189,17 +227,19 @@ class User:
                 'id': self.primary_group.reg_info()['id']
             },
             'attributeStore': {
-                self.ATTR_USERNAME: self.info.username,
+                self.ATTR_USERNAME: self.force_username or self.info.username,
                 self.ATTR_ORG_ID: CONFIG['backend.bwidm']['org_id'],
             }
         })
+        # if the user is already registered for the service, we're done here
+        if self._is_registered():
+            return
 
         old_reg = get_active_reg_info(self.info.unique_id)
-
         # We wait until the 'lastReconciled' timestamp changes, which means that our update was sucessfully deployed
         reg = old_reg
         while reg['lastReconcile'] == old_reg['lastReconcile']:
-            sleep(0.1)
+            sleep(0.3)
             logger.debug("Received registration reconciled at {}. That is not up-to-date. Checking again.".format(
                 reg['lastReconcile']))
 
@@ -223,12 +263,6 @@ class User:
         """Deregister the user from the given service in BWIDM."""
         BWIDM.get('external-reg', 'deregister', 'externalId', self.info.unique_id,\
                   'ssn', CONFIG['backend.bwidm.service']['name'])
-        # # FIXME: This is a silly workaround, to make sure, user is really # deleted
-        # from time import sleep
-        # sleep(0.5)
-        # BWIDM.get('external-reg', 'deregister', 'externalId', self.info.unique_id,\
-        #           'ssn', CONFIG['backend.bwidm.service']['name'])
-        # # FIXME: End of (this) silly workaround
 
     def deactivate(self):
         """Deactivate the user, this does not delete from BWIDM, but sets
@@ -265,8 +299,11 @@ class User:
             if to_be_added_to:
                 logger.info('Add user {} to groups {}'.format(
                     self.info.username, ",".join(g['name'] for g in to_be_added_to)))
+            grp_add_retvals = []
             for grp in to_be_added_to:
-                BWIDM.get('group-admin', 'add', 'groupId', grp['id'], 'userId', reg_info['id'])
+                grp_add_retvals.append(BWIDM.get('group-admin', 'add', 'groupId', grp['id'], 'userId', reg_info['id'], fail=False))
+            if len(grp_add_retvals) > 0:
+                logger.debug(F"Group add retvals: {grp_add_retvals}")
 
     def install_ssh_keys(self):
         self.external_user_update({
@@ -307,9 +344,24 @@ class User:
             if new_state[k] is None:
                 new_state[k] = {}
 
+        try:
+            formatted_json = (json.dumps(state_updates, sort_keys=True, indent=4, separators=(',', ': ')))
+            logger.debug("  logging state_updates to jsonlog")
+            jsonlogger.debug(F"state_updates:  {formatted_json}")
+
+            formatted_json = (json.dumps(current_state, sort_keys=True, indent=4, separators=(',', ': ')))
+            logger.debug("  logging current_state_updates to jsonlog")
+            jsonlogger.debug(F"current_state: {formatted_json}")
+
+            logger.debug("  logging new_to jsonlog")
+            formatted_json = (json.dumps(new_state, sort_keys=True, indent=4, separators=(',', ': ')))
+            jsonlogger.debug(F"new state for regapp:  {formatted_json}")
+        except:
+            pass
         BWIDM.post('external-user', 'update', json=new_state)
 
     def reg_info(self, json=True, **kwargs):
+        # FIXME: Cache this functions results!
         rsp = BWIDM.get('external-user', 'find', 'externalId', self.info.unique_id, **kwargs)
         return rsp.json() if json else rsp.content
 
@@ -318,6 +370,8 @@ class Group:
         self.name = name
 
     def exists(self):
+        # FIXME: Group existence needs to be checked with using also
+        # CONFIG['backend.bwidm.service']['name']
         return b'no such group' not in BWIDM.get('group-admin', 'find', 'name', self.name, fail=False).content
 
     def create(self):
